/*! Copyright (c) 2019 Analog Devices, Inc.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-
INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

//=============================================================
//    Module       : bootloader.c
//    Description  : Demonstrates how to write to key flash locations
//					 			 : for kernel bootloader operation.
//								 : 1. Program the device, ensure GPIO5 is present (near LIN jumper)
//								 : 2. Green LED should flash, remove GPIO5 then reset
//								 : 3. Red LED should now flash indicating kernel jumped to bootloader
//    Date         : Jan. 2023
//    Version      : v1.00
//    Changelog    : v1.00 Initial
//	  By		       : ADI Applications
//=============================================================

// Includes
#include "ADuCM342.h"             // Component selection
#include <absacc.h>

// Flash Controller Commands
#define FLSH_PAGE_ERASE        		0x1
#define FLSH_SIGN           			0x2
#define FLSH_WRITE          			0x4

// Flash Controller FEESTA Status
#define CMD_BUSY        					0x1
#define CMD_COMPLETE        			0x4

// User key for the flash controller
#define USER_KEY_1 								0xF456
#define USER_KEY_2 								0xF123

// Prototypes
void GPIO_INIT(void);
void DELAY(unsigned int);
void FLASH_Word_Write ( unsigned int ,unsigned int, unsigned int );
void FLASH_Erase(unsigned int);
void TOG_P3(void);
extern void TOG_P2(void);

// Global variables
unsigned char dummy_var;
unsigned int StatusReg, my_FEESIG;
//const unsigned int iPage0CS[2] __at(0x07f8) = {0xffffffff,0x16400000};	// for first startup vector Kernel to user code

//==========================================================================================
//										main
//==========================================================================================
int main (void)
{
 unsigned int lcl_RSTSTA;

 // Initialise
 T3CON = 0;																	// Turn off watchdog
 DELAY(4000000);														// Just in case for debug access !
 GPIO_INIT();

 lcl_RSTSTA = RSTSTA;									  		// Read reset status register
 if(lcl_RSTSTA & 0x9)												// If first POR then bits 0,3 should be set.
 {
  RSTSTA = 0x0F;														// Clear Reset status bits.

  FLASH_Erase(0x1F800);											// Erase last flash page in preparation for programming

  // 128k M331 - First write bootloader entry point
  FLASH_Word_Write(0x000007F8,0xFFFFFFFF, 0x00000000);			// Corrupt the key value so on next reset, user code does not get vectored
  FLASH_Word_Write(0x0001FFE0,0x00018801, 0x00018800);

  // Get CRC of bootloader area, could also use the key 0x16400000 value also
  FEEKEY = 0xF456;
  FEEKEY = 0xF123;
  FEEADR1L = 0x00018800;										// Page start for Sign
  FEEADR1H = 0x0001FFFF;										// Page end for Sign
  FEECMD = FLSH_SIGN;												// Run signature command

  do{
   do{
    StatusReg = FEESTA;
	 }while(StatusReg & CMD_BUSY);
  }while(!(StatusReg & CMD_COMPLETE));
  my_FEESIG = FEESIGN;												// Store signature before writing it to flash
  // Next write the CRC and low address
  FLASH_Word_Write(0x0001FFF8,0xFFFFFFFF, my_FEESIG); 	    	// Comment in to use the CRC
  // Or could just write a defined key instead
  //FLASH_Word_Write(0x0001FFF8,0xFFFFFFFF, 0x16400000);  		// Comment in to use the Key
 }

 while (1)
 {
  TOG_P3();
  dummy_var = 0;
  if(dummy_var > 1)
  {																					// Should never execute,
   TOG_P2();  															// Needed so compiler will not optimise out the other C file.
  }
 }
}

//==========================================================================================
//				Simple delay routine
//==========================================================================================
void DELAY(unsigned int length)
{
 while (length>1)
 {
 length--;
 }
}

//==========================================================================================
//				Init GPIOs
//==========================================================================================
void GPIO_INIT(void)
{
 GP0CON = 0x00;	   												// Enable as GPIOs
 GP0PUL = 0x3F;													// Activate pull ups
 GP0OEN = 0x3F;													// Enable GPIO 2,3
}

//==========================================================================================
//				Toggle P3 LED
//==========================================================================================
void TOG_P3(void)
{
 static int count = 0;

 GP0TGL = 0x08;
 if(count <= 150)												// Check if its time to reset the device
 {
  count ++;
  DELAY(100000);
 }
 else
 {
  AIRCR = 0x05FA0004;											// execute a Software Reset !!
  while(1);
 }
}

//==========================================================================================
//				Flash Word Write - 64 bits Must be Written to
//==========================================================================================
void FLASH_Word_Write (unsigned int nAddress, unsigned int Data_Low, unsigned int Data_High )
{
 volatile unsigned int uiSTA;							// Variable for FEESTA

 FEEADR = nAddress;
 FEEDATL = Data_Low;
 FEEDATH = Data_High;

 FEECMD = FLSH_WRITE;											// Write to the FLASH
 do{
  do																			// 	Write the command to erase a page
  {
   StatusReg = FEESTA;
  }while(StatusReg & CMD_BUSY);
 }while(!(StatusReg & CMD_COMPLETE));
}

//==========================================================================================
//				Erases a page in Flash. Takes word aligned address.
//==========================================================================================
void FLASH_Erase(unsigned int nAddress)
{
 volatile unsigned int StatusReg;

 FEEKEY = USER_KEY_1;											// Enter Key
 FEEKEY = USER_KEY_2;

 FEEADR1L = nAddress;											// 	Enter any address in the page
 FEECMD = FLSH_PAGE_ERASE;
 do{
  do																			// 	Write the command to erase a page
  {
   StatusReg = FEESTA;
  }while(StatusReg & CMD_BUSY);
 }while(!(StatusReg & CMD_COMPLETE));
}

//=====================================================================
// 						LINint0 Normal LIN Event handler
//=====================================================================
void LIN0_Int_Handler (void)
{
}

//=====================================================================
// 						LINint1 Error Event handler
//=====================================================================
void LIN1_Int_Handler (void)
{
}

//=====================================================================
// 						LINint2 Sleep / Wake Event handler
//=====================================================================
void LIN2_Int_Handler (void)
{
}

//=====================================================================
// 						SYSTICK Handler
//=====================================================================
void SysTick_Int_Handler(void)
{
}

//=====================================================================
// 						WUP Timer Interrupt Handler
//=====================================================================
void WakeUp_Int_Handler(void)
{
}

//=====================================================================
// 						GP Timer Interrupt Handler
//=====================================================================
void GP_Tmr_Int_Handler(void)
{
}

//=====================================================================
// 						ADC Interrupt Handler
//=====================================================================
void ADC_Int_Handler(void)
{
}

//==========================================================================================
//				Hard Fault Handler, 2bit or >2bit ECC error interrupt
//==========================================================================================
void Hard_Fault_Exception(void)
{
}

