/*! Copyright (c) 2019 Analog Devices, Inc.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-
INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

/*****************************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : November. 2019

 File          : I-ADC_AVDD18_136.c

 Hardware      : Applicable to ADuCM33xWFS

 Description   : This s/w performs ADC conversions on I-ADC and V-ADC.
								 The s/w will perform frequently measuements using the internal
								 voltage source AVDD18/136 at the I-ADC channel
				 
*****************************************************************************/

/****************************************************************************/
/*		Includes                                                              */
/****************************************************************************/
#include "ADuCM342.h"             // Component selection
#include <absacc.h>

/****************************************************************************/
/*		Local Defines                                                         */
/****************************************************************************/

// Definitions
#define INT_TEMP_OF (*(volatile short *) 0x000207E8)
#define INT_TEMP_GN (*(volatile short *) 0x000207EA)

#define VREF_V		1.2
#define FS 				0x0FFFFF00
#define TEMPCO		0.000274
#define SHUNT_R	0.0001


/****************************************************************************/
/*		Local Macros                                                         */
/****************************************************************************/

/****************************************************************************/
/*		Type declarations                                                     */
/****************************************************************************/
typedef enum		// ADC channels
{
	ADC_I	= (1<<0),	
	ADC_V	= (1<<1),	
	ADC_T	= (1<<2)	
}ADC_CHANNEL;

typedef enum		// ADC channels
{
	GAIN1		= 0x0,	
	GAIN2		= 0x1,	
	GAIN4		= 0x2,	
	GAIN8		= 0x3,	
	GAIN16	= 0x4,	
	GAIN32	= 0x5,	
	GAIN64	= 0x6,	
	GAIN128	= 0x7,	
	GAIN256	= 0x8,	
	GAIN512	= 0x9
}ADC_PGA;

typedef enum    // ADCFTL settings as per datasheet 
{
  ADC_F1000_CHOPY =  0x08101,
  ADC_F1000_CHOPN =  0x00007,   
}ADC_FLT_SETTINGS;

typedef enum    // GPIO Pins
{
	PIN0 = (1<<0),
	PIN1 = (1<<1),
	PIN2 = (1<<2),
	PIN3 = (1<<3),
	PIN4 = (1<<4),
	PIN5 = (1<<5)
} GPIO_PINS;

	/****************************************************************************/
/*		Constants                                                             */
/****************************************************************************/
const unsigned int iPage0CS[2] __at(0x07f8) = {0xffffffff,0x16400000};	
// for proper startup from Kernel 

/****************************************************************************/
/*		Prototypes                                                            */
/****************************************************************************/
void 		Wait (unsigned int);
void 		GPIO_INIT(void);
void 		VREF_INIT(void);
void 		ADC_INIT(void);
void 		INIT_TEMP_SENS(void);
void 		IADC_ConfigChange(unsigned int);

/****************************************************************************/
/*		Global Variables                                                      */
/****************************************************************************/
signed int 			i32IADC; 
unsigned int		u32VTADC;
unsigned short 	u16ADCSTA;
float 					fVADC_VOLT, fIADC_VOLT;
float 					fVbat, fIamp, fAVDD136;
unsigned char		u32ConvCnt = 1;

unsigned int 		ADC = (1<<5);		// NVIC interrupt
unsigned char		bConfigChanged;	// Short measurement in progress flag 
unsigned int		u32Buffer;

/****************************************************************************/
/*		Main Function                                                         */
/****************************************************************************/
int main(void)
{
// disable WDG timer
	T3CON = 0;
	
// initialise GPIOs
	GPIO_INIT();
// Initialize Vref
	VREF_INIT();
// Initialize ADC
	ADC_INIT();

// Enable Interrupts	
	ISER0  =(ADC);	
  while (1) 
	{
	}
}

//============================================================================
//	ADC Interrupt Handler
//============================================================================
void ADC_Int_Handler (void)
{
  u16ADCSTA = ADCSTA;				// get ADCSTA
	
  u32VTADC 	= ADC1DAT;			// read VT_ADC
  i32IADC 	= ADC0DAT;			// read IADC
	
 	GP0CLR = (PIN1);					// indicate interrupt

	if (u16ADCSTA & ADC_V)		// VADC interrupt
	{
   	GP0CLR = (PIN3);				// indicate V-ADC
  	Wait(10);								// give time to indicate
		fVADC_VOLT = u32VTADC * VREF_V / FS;	// ADC input Voltage
		fVbat	=	fVADC_VOLT *	24;  						// V
   	GP0SET = (PIN3);				// indicate V-ADC
	}
	if (u16ADCSTA & ADC_I)		// IADC interrupt
  {
   	GP0CLR = (PIN2);				// indicate I-ADC
  	Wait(10);								// give time to indicate

		if (bConfigChanged)
		{
			fAVDD136 = i32IADC * VREF_V / FS;			// ADC input Voltage
			bConfigChanged = 0;		// clear ConfigChanged flag
			ADC_INIT();						// back to normal configuration
		}
		else
		{
			fIADC_VOLT = i32IADC * VREF_V / FS;			// ADC input Voltage
			fIamp	= fIADC_VOLT * 1000 / SHUNT_R;    // I (mA)
		}
    GP0SET = (PIN2);		    // indicate I-ADC

		if ((u32ConvCnt % 0x20) == 0)
		{
			GP0CLR = (PIN4);		  // indicate settings change
			u32Buffer = ADC0CON & (~(7<<6)); 	// clear ADC0CH
			ADC0CON = u32Buffer | (2<<6);			// set AVDD18/136 in ADC0CH
			IADC_ConfigChange (u32Buffer);
			bConfigChanged = 1; 	// set ConfigChanged flag
			GP0SET = (PIN4);		  // indicate settings change
		}
	}
  u32ConvCnt++;	//	increment conversion counter			
	GP0SET = (PIN1);					// indicate interrupt
}

//============================================================================
//	simple wait function
//============================================================================
void Wait (unsigned int u32Cnt)
{		
 	while (u32Cnt--){};
}
//============================================================================

//============================================================================
//	Initialise the GPIOs
//============================================================================
void GPIO_INIT(void)
{
	GP0CON =  ((0<< 10)  // P0.5 
            |(0<< 8)   // P0.4 
            |(0<< 6)   // P0.3 
            |(0<< 4)   // P0.2
            |(0<< 2)   // P0.1 
            |(0<< 0)   // P0.0 
            );
	GP0PUL = 0x3F;			// Activate pull ups
	GP0SET = 0x3F;			// set high 
	GP0OEN = ((0<< 5)   // P0.5 
           |(1<< 4)   // P0.4 
           |(1<< 3)   // P0.3 
           |(1<< 2)   // P0.2
           |(1<< 1)   // P0.1 
           |(0<< 0)   // P0.0 
            );
}
//============================================================================

//============================================================================
//	Initialise the ADCs
//============================================================================
void VREF_INIT(void)
{
	// Reference Settings
	HRFCTRL 	= 0x00;			// Auto mode for reference.
	IRFPD		 	= 0x00;			// Enable the internal reference buffer
 
  Wait(0x800);          // wait a min. of 500us (see DS for details)
}

//============================================================================
//	Initialise the ADCs
//============================================================================
void ADC_INIT(void)
{
	ADCMDE 		= 0x03;			// set to idle mode
	ADC0CON		=	(0x80C00 	// IADC enabled; PGA scale =0b11 (off); 
						   |GAIN8 	// Gain=8;
						  );	 
	ADC1CON		=	0x80200;		// VADC enabled; Unipolar
	ADCFLT  	=	ADC_F1000_CHOPY;
	ADCCFG		= 0x0;
	ADCMSKI		=(ADC_I
						 |ADC_V
						);					// I-V-T ADC interrupt enabled
	ADCMDE 		= 0x01;			// contineous conversion mode
}
//============================================================================

//====================================================================
//					  Config I-ADC
//====================================================================
void IADC_ConfigChange(unsigned int ADC0_CON)
{
	ADCMDE 		= 0x03;			// set to idle mode
	ADC0CON 	= ADC0_CON; // set ADC0 Configuration
	ADCMDE 		= 0x01;			// contineous conversion mode
}
//====================================================================
//					  LINint0 Handler
//====================================================================
void LIN0_Int_Handler (void)
{
}

//====================================================================
//					  LINint1 Handler
//====================================================================
void LIN1_Int_Handler (void)
{
}

//====================================================================
//					  LINint2 Handler
//====================================================================
void LIN2_Int_Handler (void)
{
}

//============================================================================
//				SYSTICK Handler
//============================================================================
void SysTick_Int_Handler(void)
{
}

//=====================================================================
// 						WUP Timer Interrupt Handler
//=====================================================================
void WakeUp_Int_Handler(void)
{
}

//=====================================================================
// 						GPTimer Interrupt Handler
//=====================================================================
void GP_Tmr_Int_Handler(void)
{
}

//==========================================================================================
//				Hard Fault Handler, 2bit or >2bit ECC error interrupt
//==========================================================================================
void Hard_Fault_Exception(void)
{
}


// end off file
