/*! Copyright (c) 2019 Analog Devices, Inc.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-
INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

//===========================================================================
//    Module       : main.c
//    Description  : Demonstrates how to configure the M342 for powerdown
//								 : Sets up the LIN to powerdown the device after 4s. LIN Wake will wake the device
//								 : Ensure device has debugger disconnected, a 1k pull up between LIN and Vdd. all GPIO jumpers disconnected
//								 : As well ensure device has been fully reset by a POR.
//    Date         : Jan. 2023
//    Version      : v1.00
//    Changelog    : v1.00 Initial
//	  By		       : ADI Apps
//===========================================================================

// Includes
#include "ADuCM342.h"             // Component selection
#include <absacc.h>

// Definitions
#define FALSE 			0
#define TRUE 				1

// HV status info 
#define BUSY				(1<<0)
#define PARI				(1<<1)
#define ACK					(1<<2)

// HV read/write commands
#define HV_RD_CGF		0
#define HV_RD_STA		2
#define HV_RD_REG		5
#define HV_WR_CFG		8

// Prototypes
void 						GPIO_INIT(void);
unsigned short 	HV_Comms (unsigned char data, unsigned char cmd);

// Globals
unsigned char Sleep = FALSE;

// Constants
const unsigned int iPage0CS[2] __at(0x07f8) = {0xffffffff,0x16400000};	// for direct startup from Kernel
//============================================================================
//				main
//============================================================================
int main (void)
{
	unsigned int i;
	T3CON = 0;  								// Turn off watchdog

	GPIO_INIT();
	ADC0CON = 0x80000;					// Turn on ADCs
	ADC1CON = 0x80000;
	HRFCTRL = 0x00;						  // Ensure ADCs automatically control the reference
	IRFPD = 0x00;								// Enable the buffer

	ADC0CON = 0x30000;					// Turn ADCs off
	ADC1CON = 0x30000;

	LINCON = 0x1101;						// LIN logic on, sleep counter on
	ISER0 = 0x400;							// NVIC Interrupt on a Wake / Sleep condition

	HV_Comms (0x05, HV_WR_CFG);	// Enable Top die LIN Transceiver
	while(1)
	{
		if(Sleep)
		{
			SCR = 0x04;										// Enable deep sleep mode
			PWRKEY = 0x4859;
			PWRKEY = 0xF27B;								// Prekeys
			PWRMOD = 0x05;									// Write Hibernate to Power Mode,
			for(i=0;i<1;i++){}							// small delay
			__DSB();
			__wfi();
		}
	}
}

//============================================================================
//				HV Die Communication 
//============================================================================
unsigned short HV_Comms(unsigned char data, unsigned char cmd)
{
	unsigned char CommsSta; 
	
  do{
		HVDAT = data;									// write data (irrelevant for a read)
		HVCON = cmd;									// write command
		do{
			CommsSta = HVCON;
		}while(CommsSta & BUSY);			// Wait for busy to clear
	}while(!(CommsSta & ACK));			// Ensure Ack is set	
	return HVDAT;
	// note that upper HVDAT nipple contains last excecuted command
	//					 lower Byte contains the data read.
}


//============================================================================
//				GPIO Init
//============================================================================
void GPIO_INIT(void)
{
 GP0CON = 0x00;	   						// Enable as GPIOs
 GP0OEN = 0x00;								// Enable GPIO 0 as i/p
 GP0PUL = 0x3F;								// Pull ups enabled
}

//=====================================================================
// 						LINint0 Normal LIN Event handler
//=====================================================================
void LIN0_Int_Handler (void)
{
}

//=====================================================================
// 						LINint1 Error Event handler
//=====================================================================
void LIN1_Int_Handler (void)
{
}

//=====================================================================
// 						LINint2 Sleep / Wake Event handler
//=====================================================================
void LIN2_Int_Handler (void)
{
	unsigned int LIN_STA;

	LIN_STA = LINSTA;
	if(LIN_STA & 0x80)						// 4s Inactivity
	{
		LINSTA = 0x80;							// Clear the status flag
		Sleep = TRUE;
	}
	if(LIN_STA & 0x100)	   				// Wake
	{
		HV_Comms (0x05, HV_WR_CFG);	// Enable Top die LIN Transceiver
		LINSTA = 0x100;							// Clear the status flag
		Sleep = FALSE;
	}
}

//============================================================================
//				SYSTICK Handler
//============================================================================
void SysTick_Int_Handler(void)
{
}

//=====================================================================
// 						WUP Timer Interrupt Handler
//=====================================================================
void WakeUp_Int_Handler(void)
{
}

//=====================================================================
// 						GP Timer Interrupt Handler
//=====================================================================
void GP_Tmr_Int_Handler(void)
{
}

//=====================================================================
// 						ADC Interrupt Handler
//=====================================================================
void ADC_Int_Handler(void)
{
}

//==========================================================================================
//				Hard Fault Handler, 2bit or >2bit ECC error interrupt
//==========================================================================================
void Hard_Fault_Exception(void)
{
}

