/*! Copyright (c) 2019 Analog Devices, Inc.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-
INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

//=============================================================
//    Module       : main.c
//    Description  : Demonstrates how to setup and configure the internal
//				   			   internal temperature sensor.
//				   			   Use debug mode to pause at TempDataC value
//    Date         : Jan. 2023
//    Version      : v1.00
//    Changelog    : v1.00 Initial
//	  By		   : ADI Apps
//=============================================================

// Includes
#include "ADuCM342.h"             // Component selection
#include <absacc.h>

// wait function
void Wait(unsigned int Cnt){while(Cnt--);}    

// Definitions
#define INT_TEMP_OF (*(volatile short *) 0x000207E8)
#define INT_TEMP_GN (*(volatile short *) 0x000207EA)

// Prototypes
void Vref_Init(void);
void ADC_Init(void);
void ADC_Start(void);

// Globals
unsigned int T_ADCDATA, ADC2OF_BUF, ADC2GN_BUF;
float TempDataV,TempDataK,TempDataC = 0x0000;						// Float only used for demonstration.
const unsigned int iPage0CS[2] __at(0x07F8) = {0xFFFFFFFF,0x16400000};	// for startup from Kernel to user code

//=====================================================================
// 						main
//=====================================================================
int main (void)
{
 T3CON = 0;  						// Turn off watchdog

 Vref_Init();           // Enable the 1.2v reference first
 ADC_Init();						// Perform ADC0 Init
 ISER0 = 0x20;					// Enable ADC Interrupt in the NVIC

 ADC_Start(); 					// Start
 while(1)
 {
 }
}

//==============================================================================================
// 						ADC Interrupt Handler - uses floating point
//==============================================================================================
void ADC_Int_Handler(void)
{
 unsigned int my_ADCSTA;

 my_ADCSTA = ADCSTA;
 if(my_ADCSTA & 0x04)						// T Data
 {
  T_ADCDATA = ADC1DAT;
  TempDataV = T_ADCDATA	* 4.47e-9;
  TempDataK = TempDataV / 274e-6;
  TempDataC = TempDataK - 273.15;
 }
}

//====================================================================
//					  ADC Config
//====================================================================
void Vref_Init(void)
{
 // Enable the 1.2v reference first
 HRFCTRL = 0x00;							// Auto mode for reference.
 IRFPD = 0x00;								// Enable the internal reference buffer
 
 Wait(0x800);                 // wait a min. of 500us (see DS for details)
}
//====================================================================
//					  ADC Config
//====================================================================
void ADC_Init(void)
{
 // set Idle Mode
 ADCMDE =  ( 0 << 6 )					// Reserved
					|( 0 << 5 )					// Always Interrupt off
					|( 0 << 4 )					// Reserved
					|( 0 << 3 )					// ADC Normal Mode
					|( 3 << 0 );				// ADC Idle Mode

 // I Channel not used
 ADC0CON = 0x030000;

 // Buffer default coefficients for later use (ext. temp. sensing) 
 ADC2GN_BUF = ADC2GN;
 ADC2OF_BUF = ADC2OF;

 // set coefficients for internal temperature sensing
 ADC2OF = INT_TEMP_OF;    // Copy calibrated MANID value into Temperature Offset register
 ADC2GN = INT_TEMP_GN;    // Copy calibrated MANID value into Temperature Gain register

 // T Channel setting
 ADC1CON = ( 1 << 19 )						// Enable
					|( 0 << 18 )						// Reserved
					|( 0 << 16 )						// No bypass
					|( 0 << 14 )						// Reserved
					|( 0 << 12 )						// Current Sources off
					|( 0 << 10 )						// Reserved
					|( 1 << 9 )							// Code Unipolar
					|( 2 << 6 )							// Mux select (int. temp. sensing)
					|( 0 << 5 )							// Reserved
					|( 0 << 4 )							// Internal 1.2v
					|( 0 << 0 );						// Reserved

 ADCMSKI = ( 0 << 7 )							// Reserved
					|( 0 << 6 )							// ADCINT Off
					|( 0 << 5 )							// ADC0ATHEX Interrupt Off
					|( 0 << 4 )							// ADC0THEX Interrupt Off
					|( 0 << 3 )							// ADC0OVR Interrupt Off
					|( 1 << 2 )							// ADC2RDY Interrupt On
					|( 0 << 1 )							// ADC1RDY Interrupt Off
					|( 0 << 0 );						// ADC0RDY Interrupt Off

 ADCCFG =  ( 0 << 7 )							// 60k Switch Enable off
					|( 0 << 6 )							// Reserved
					|( 0 << 5 )							// ADC0 Accululator Enable off
					|( 0 << 3 )							// ADC0 Comparator Enable off
					|( 0 << 2 )							// ADC0 Over Range Enable off
					|( 0 << 1 )							// Fast Temp Enable	initially off
					|( 0 << 0 );						// ADC Result Counter Enable off

 // Filter Settings	 1kHz Chop on
 ADCFLT =  ( 0 << 16 )						// Sinc3
					|( 1 << 15 )						// Chop
					|( 0 << 14 )						// Running Average by 2	off
					|( 1 << 8 )							// AF
					|( 0 << 7 )							// Notch2 Enable off
					|( 1 << 0 );						// SF // V/T Channel	config
}

//=====================================================================
// 						Start ADC Conversion
//=====================================================================
void ADC_Start(void)
{
 ADCMDE = 0x1;								// Cont conversion on any ADC enabled
}

//=====================================================================
// 						LINint0 Normal LIN Event handler
//=====================================================================
void LIN0_Int_Handler (void)
{
}

//=====================================================================
// 						LINint1 Error Event handler
//=====================================================================
void LIN1_Int_Handler (void)
{
}

//=====================================================================
// 						LINint2 Sleep / Wake Event handler
//=====================================================================
void LIN2_Int_Handler (void)
{
}

//============================================================================
//				SYSTICK Handler
//============================================================================
void SysTick_Int_Handler(void)
{
}

//=====================================================================
// 						WUP Timer Interrupt Handler
//=====================================================================
void WakeUp_Int_Handler(void)
{
}

//=====================================================================
// 						GPTimer Interrupt Handler
//=====================================================================
void GP_Tmr_Int_Handler(void)
{
}

//==========================================================================================
//				Hard Fault Handler, 2bit or >2bit ECC error interrupt
//==========================================================================================
void Hard_Fault_Exception(void)
{
}

