/*! Copyright (c) 2019 Analog Devices, Inc.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-
INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF
CLAIMS OF INTELLECTUAL PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*****************************************************************************/

/*****************************************************************************

 Author        : ADI - Apps            www.analog.com/MicroConverter

 Date          : Jan. 2023

 File          : V-ADC_Diag_I_Source.c

 Hardware      : Applicable to ADuCM342

 Description   : This s/w performs ADC conversions on I-ADC. V-ADC
								 The s/w will perform frequent measuements using the internal
								 diagnostic current source for the main attenuator (Vbat) 
								 available on the High Voltage die.
*****************************************************************************/

/****************************************************************************/
/*		Includes                                                              */
/****************************************************************************/
#include "ADuCM342.h"             // Component selection
#include <absacc.h>


/****************************************************************************/
/*		Local Defines                                                         */
/****************************************************************************/

// Definitions
#define INT_TEMP_OF (*(volatile short *) 0x000207E8)
#define INT_TEMP_GN (*(volatile short *) 0x000207EA)

#define VREF_V		1.2
#define FS 				0x0FFFFF00
#define TEMPCO		0.000274
#define SHUNT_R		0.0001


/****************************************************************************/
/*		Local Macros                                                         */
/****************************************************************************/

/****************************************************************************/
/*		Type declarations                                                     */
/****************************************************************************/
typedef enum		// ADC channels
{
	ADC_I	= (1<<0),	
	ADC_V	= (1<<1),	
	ADC_T	= (1<<2)	
}ADC_CHANNEL;
typedef enum		// ADC channels
{
	GAIN1		= 0x0,	
	GAIN2		= 0x1,	
	GAIN4		= 0x2,	
	GAIN8		= 0x3,	
	GAIN16	= 0x4,	
	GAIN32	= 0x5,	
	GAIN64	= 0x6,	
	GAIN128	= 0x7,	
	GAIN256	= 0x8,	
	GAIN512	= 0x9
}ADC_PGA;

typedef enum    // ADCFTL settings as per datasheet 
{
  ADC_F1000_CHOPY =  0x08101,
  ADC_F1000_CHOPN =  0x00007,   
}ADC_FLT_SETTINGS;

typedef enum    // GPIO Pins
{
	PIN0 = (1<<0),
	PIN1 = (1<<1),
	PIN2 = (1<<2),
	PIN3 = (1<<3),
	PIN4 = (1<<4),
	PIN5 = (1<<5)
} GPIO_PINS;

	/****************************************************************************/
/*		Constants                                                             */
/****************************************************************************/
const unsigned int iPage0CS[2] __at(0x07f8) = {0xffffffff,0x16400000};	
// for proper startup from Kernel 

/****************************************************************************/
/*		Prototypes                                                            */
/****************************************************************************/
void 						Wait (unsigned int);
void 						GPIO_INIT(void);
void            VREF_INIT(void);
void 						ADC_INIT(void);
void 						INIT_TEMP_SENS(void);
unsigned short 	HV_COM(unsigned char, unsigned char);

/****************************************************************************/
/*		Global Variables                                                      */
/****************************************************************************/
signed int 			i32IADC; 
unsigned int		u32VTADC;
unsigned short 	u16ADCSTA;
unsigned short 	u16ADC2GN, u16ADC2OF; 	// temp. channel oeff. buffers
float 					fVADC_VOLT, fIADC_VOLT, fTADC_VOLT;
float 					fVbat, fIamp, fVIsource;
unsigned int		u32ConvCnt = 1;
unsigned char		bIsource = 0;				// diag attenuator flag

unsigned int 		ADC = (1<<5);		// NVIC interrupt

/****************************************************************************/
/*		Main Function                                                         */
/****************************************************************************/
int main(void)
{
// disable WDG timer
	T3CON = 0;
// initialize GPIOs
	GPIO_INIT();	
// Initialize Vref and ADC
  VREF_INIT();
	ADC_INIT();
  INIT_TEMP_SENS();

// Enable Interrupts	
	ISER0  =(ADC);	
 while (1) 
	{
	}
}

//============================================================================
//	simple wait function
//============================================================================
void Wait (unsigned int u32Time)
{		
 	while ((u32Time)--){};
}
//============================================================================

//============================================================================
//	Initialise the GPIOs
//============================================================================
void GPIO_INIT(void)
{
	GP0CON =  ((0<< 10)  // P0.5 
            |(0<< 8)   // P0.4 
            |(0<< 6)   // P0.3 
            |(0<< 4)   // P0.2
            |(0<< 2)   // P0.1 
            |(0<< 0)   // P0.0 
            );
	GP0PUL = 0x3F;		// Activate pull ups
	GP0OEN = ((0<< 5)   // P0.5  (GPIO5)
           |(1<< 4)   // P0.4 
           |(1<< 3)   // P0.3 
           |(1<< 2)   // P0.2
           |(1<< 1)   // P0.1 
           |(0<< 0)   // P0.0 
            );
}
//============================================================================

//============================================================================
//	Initialise Internal Temp Sensor
//============================================================================
void INIT_TEMP_SENS(void)
{
  unsigned short Temp;
	
	// buffer default Gain and Offset coefficients (external temp. sensing)
	u16ADC2GN = ADC2GN;
	u16ADC2OF = ADC2OF;
  
  Temp 		= INT_TEMP_OF;						
  ADC2OF 	= Temp;	// Copy calibrated MANID value into Temperature Offset register
  Temp 		= INT_TEMP_GN;
  ADC2GN 	= Temp;	// Copy calibrated MANID value into Temperature Gain register
}

//============================================================================
//	Initialise Vref
//============================================================================
void VREF_INIT(void)
{
	// Reference Settings
	HRFCTRL 	= 0x00;			// Auto mode for reference.
	IRFPD		 	= 0x00;			// Enable the internal reference buffer
 
  Wait(0x800);          // wait a min. of 500us (see DS for details)
}

//============================================================================
//	Initialise the ADCs
//============================================================================
void ADC_INIT(void)
{
	// Reference Settings
	ADCMDE 		= 0x03;			// set to idle mode
	ADC0CON		=	(0x80C00 	// IADC enabled; PGA scale =0b11 (off); 
						   |GAIN8 	// Gain=8;
						  );	 
	ADC1CON		=	0x80200;		// VADC enabled; Unipolar
	ADCFLT  	=	ADC_F1000_CHOPY;
	ADCCFG		= 0x0;
	ADCMSKI		=(ADC_I
						 |ADC_V
						 |ADC_T
						);					// I-V-T ADC interrupt enabled
	ADCMDE 		= 0x01;			// contineous conversion mode
}
//============================================================================

//============================================================================
//	ADC Interrupt Handler
//============================================================================
void ADC_Int_Handler (void)
{
  u16ADCSTA = ADCSTA;				// get ADCSTA
	
  u32VTADC 	= ADC1DAT;			// read VT_ADC
  i32IADC 	= ADC0DAT;			// read IADC
	
 	GP0TGL = (PIN1);					// indicate interrupt

	if (u16ADCSTA & ADC_V)		// VADC interrupt
	{
   	GP0TGL = (PIN3);				// indicate V-ADC
  	Wait(10);								// give time to indicate
		fVADC_VOLT = u32VTADC * VREF_V / FS;	// ADC input Voltage
		
		if (bIsource)
		{
			fVIsource = fVADC_VOLT * 24;		// (V);
			HV_COM(0x0, 0x8);			// clear HVCFG0
			ADC1CON |= ADC1CON;		// restart ADC1
			bIsource = 0; 				// clear short flag 
		}
		else
		{
			fVbat	 =	fVADC_VOLT * 24;	// (V)
		}
		// see DS specification for value of (fVIsource - fVbat)

    if (u32ConvCnt%0x100 == 0)	
		{
			HV_COM((1<<3), 0x8);	// set ASEL in HVCFG0
			ADC1CON |= ADC1CON;		// restart ADC1
			bIsource   = 1;						// set short measurement flag
		}	
  	GP0TGL = (PIN3);				// indicate V-ADC
}
	if (u16ADCSTA & ADC_I)		// IADC interrupt
  {
   	GP0TGL = (PIN2);				// indicate I-ADC
  	Wait(10);								// give time to indicate
    fIADC_VOLT = i32IADC * VREF_V / FS;			// ADC input Voltage
		fIamp	= fIADC_VOLT * 1000 / SHUNT_R;    // mA
    GP0TGL = (PIN2);		    // indicate I-ADC
	}
  u32ConvCnt++;	//	increment conversion counter			
	GP0TGL = (PIN1);					// indicate interrupt
}

//=====================================================================
//  HV communication function						
//=====================================================================
#define BUSY	(1<<0)
#define PARI	(1<<1)
#define ACK		(1<<2)

unsigned short HV_COM(unsigned char DATA, unsigned char CMD)
{
	unsigned char HV_STA = 0;
	unsigned char timeOut = 0x20;  // prevents while(1) condition
	
	while((HVCON & BUSY));	// ensure HV interface not busy

	HVDAT = DATA;						// Load data
	HVCON = CMD; 						// Start command 

	do{
		timeOut-- ;						// decrement timeout counter
		HV_STA = HVCON;
	}while((HV_STA & BUSY)&&(timeOut));		// wait until HV interface is finished
	
	if (timeOut == 0)
		return 0xBAAD;		// timeout occured
	
	if (CMD == 0x8)			// write command
	{
		if(!(HV_STA == 0x6)) 
			return 0x0BAD;	// fail
	}
	if (CMD == 0x0)			// read command
	{
		if(!(HV_STA == 0x2))
			return 0xBAD0; 	// fail
	}
	
	return HVDAT; 
}

//====================================================================
//					  LINint0 Handler
//====================================================================
void LIN0_Int_Handler (void)
{
}

//====================================================================
//					  LINint1 Handler
//====================================================================
void LIN1_Int_Handler (void)
{
}

//====================================================================
//					  LINint2 Handler
//====================================================================
void LIN2_Int_Handler (void)
{
}

//============================================================================
//				SYSTICK Handler
//============================================================================
void SysTick_Int_Handler(void)
{
}

//=====================================================================
// 						WUP Timer Interrupt Handler
//=====================================================================
void WakeUp_Int_Handler(void)
{
}

//=====================================================================
// 						GPTimer Interrupt Handler
//=====================================================================
void GP_Tmr_Int_Handler(void)
{
}

//==========================================================================================
//				Hard Fault Handler, 2bit or >2bit ECC error interrupt
//==========================================================================================
void Hard_Fault_Exception(void)
{
}


// end off file
